/*
 * Copyright © 2020-2023 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of gaming-core.
 *
 * Gaming-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Gaming-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * gaming-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.core.domain;

import java.util.Optional;

/**
 * Generates a valid move for a given player in a given game with a given state.
 *
 * @param <P> The type of game players.
 * @param <S> The type of game states.
 * @param <M> The type of game moves.
 */
@FunctionalInterface
public interface MoveGenerator<P extends Player<P>, S extends State<P, S>, M extends Move<P, S>> {

    /**
     * Generates a valid move. If no valid move is possible an empty {@link Optional} is returned.
     *
     * @param player The player to choose the move for.
     * @param state  The state of the game to operate on.
     * @return The generated move or an empty {@link Optional} if no valid move is possible.
     */
    Optional<M> generate(P player, S state);
}
